local Utopia = LibStub("AceAddon-3.0"):GetAddon("Utopia", true)
if (not Utopia) then
	return
end
Utopia:UpdateVersion("$Revision: 66 $")

local wowVersion = tonumber((select(2,GetBuildInfo())))

local L = LibStub("AceLocale-3.0"):GetLocale("Utopia")

-- All talent scans are done by name. No messing around with slot indexes etc.
-- Notable fields:
--		interested	- The types of player interested in seeing this icon (for the automatic icon selection)

Utopia.bloodlustID = UnitFactionGroup("player") == "Horde" and 2825 or 32182		-- Bloodlust/Heroism

-- Update for major patches:
Utopia.expectedTalentLengths = {
	PRIEST		= {28, 27, 27},
	DEATHKNIGHT	= {28, 28, 30},
	ROGUE		= {27, 28, 28},
	HUNTER		= {26, 27, 28},
	MAGE		= {30, 28, 28},
	PALADIN		= {26, 26, 26},
	WARRIOR		= {31, 27, 27},
	SHAMAN		= {25, 29, 26},
	WARLOCK		= {28, 27, 26},
	DRUID		= {28, 30, 27},
}

Utopia.buffs = {
	[L["Armor"]] = {
		icon = select(3, GetSpellInfo(48942)),
		attribute = L["Armor"],
		mode = "Mitigation",
		interested = "tank",
		spells = {
			[48942] = {										-- Devotion Aura
				class = "PALADIN",
				exclusive = "aura",
				rankLevels = {1, 10, 20, 30, 40, 50, 60, 70, 74, 79},
				amounts = {55, 160, 275, 390, 505, 620, 735, 861, 1015, 1205},
				improvedPercentagePerTalentPoint = 50/3,
				maxTalentPoints = 3,
				improved = GetSpellInfo(20138),				-- Improved Devotion Aura
				regex = true,
			},
		},
	},
	[L["Armor (%)"]] = {
		icon = select(3, GetSpellInfo(16240)),
		attribute = L["Armor"],
		mode = "Mitigation",
		interested = "tank",
		spells = {
			[16240] = {										-- Ancestral Healing
				class = "SHAMAN",
				amountPerTalentPoint = 25/3,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(16240),		-- Ancestral Healing
			},
			[15363] = {										-- Inspiration
				class = "PRIEST",
				amountPerTalentPoint = 25/3,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(15363),		-- Inspiration
			},
		},
	},
	[L["Attack Power"]] = {
		icon = select(3, GetSpellInfo(47436)),
		attribute = L["AP"],
		mode = "DPS",
		interested = "melee,tank",
		reportable = true,
		spells = {
			[47436] = {										-- Battle Shout
				class = "WARRIOR",
				exclusive = "warriorbuff",
				rankLevels = {1, 12, 22, 32, 42, 52, 60, 69, 78},
				amounts = {15, 35, 55, 85, 130, 185, 232, 305, 548},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 5,
				improved = GetSpellInfo(12861),				-- Commanding Presence
				regex = 1,
			},
			[48932] = {										-- Blessing of Might
				alternate = GetSpellInfo(27141),			-- Greater Blessing of Might
				alternateOffset = 5,						-- How many ranks to add if it's the alternate spell
				class = "PALADIN",
				exclusive = "blessing",
				rankLevels = {4, 12, 22, 32, 42, 52, 60, 70, 73, 79},
				amounts = {20, 35, 55, 85, 130, 185, 232, 306, 410, 550},
				improvedPercentagePerTalentPoint = 25/2,
				maxTalentPoints = 2,
				improved = GetSpellInfo(20045),				-- Improved Blessing of Might
				regex = 1,
			},
		},
	},
	[L["Attack Power (%)"]] = {
		icon = select(3, GetSpellInfo(19506)),
		attribute = L["AP"],
		mode = "DPS",
		interested = "melee,tank",
		spells = {
			[19506] = {										-- Trueshot Aura
				class = "HUNTER",
				spec = "Marksmanship",
				amount = 10,
				amountType = "%",
				requiredTalent = GetSpellInfo(19506),
			},
			[53138] = {										-- Abomination's Might
				class = "DEATHKNIGHT",
				spec = "Blood",
				amount = 10,
				amountType = "%",
				requiredTalent = GetSpellInfo(53138),
			},
			[30809] = {										-- Unleashed Rage
				class = "SHAMAN",
				spec = "Enhancement",
				amounts = {4, 7, 10},
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(30809),
			},
		},
	},
	["Bloodlust/Heroism"] = {
		icon = select(3, GetSpellInfo(Utopia.bloodlustID)),
		attribute = L["Haste"],
		mode = "DPS",
		spells = {
			[Utopia.bloodlustID] = {						-- Bloodlust / Heroism
				class = "SHAMAN",
				amount = 30,
				amountType = "%",
			},
		},
	},
	[L["Crit Chance (Melee)"]] = {
		icon = select(3, GetSpellInfo(17007)),
		attribute = L["Melee Crit"],
		mode = "Stats",
		interested = "melee,tank",
		spells = {
			[17007] = {										-- Leader of the Pack
				class = "DRUID",
				spec = "Feral",
				amount = 5,
				amountType = "%",
				requiredTalent = GetSpellInfo(17007),		-- Leader of the Pack
			},
			[29801] = {										-- Rampage
				class = "WARRIOR",
				spec = "Fury",
				amount = 5,
				amountType = "%",
				requiredTalent = GetSpellInfo(29801),		-- Rampage
			},
		},
	},
	[L["Crit Chance (Spell)"]] = {
		icon = select(3, GetSpellInfo(24907)),
		attribute = L["Spell Crit"],
		mode = "Stats",
		interested = "caster,healer",
		spells = {
			[24907] = {										-- Moonkin Aura
				class = "DRUID",
				spec = "Balance",
				amount = 5,
				amountType = "%",
				requiredTalent = GetSpellInfo(24858),		-- Moonkin Form
			},
			[51470] = {										-- Elemental Oath
				class = "SHAMAN",
				spec = "Elemental",
				amountPerTalentPoint = 2.5,
				maxTalentPoints = 2,
				amountType = "%",
				requiredTalent = GetSpellInfo(51470),		-- Elemental Oath
			},
		},
	},
	[L["Damage (%)"]] = {
		icon = select(3, GetSpellInfo(34460)),
		attribute = L["Damage Modifier"],
		attributeShort = L["Damage"],
		mode = "DPS",
		interested = "melee,caster",
		spells = {
			[34460] = {										-- Ferocious Inspiration
				class = "SHAMAN",
				spec = "Enhancement",
				amountPerTalentPoint = 1,
				maxTalentPoints = 3,
				amountType = "%",
				temp = true,
				requiredTalent = GetSpellInfo(34460),
			},
			[31869] = {										-- Sanctified Retribution
				class = "PALADIN",
				spec = "Retribution",
				amount = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(31869),
			},
		},
	},
	[L["Damage Reduction (%)"]] = {
		icon = select(3, GetSpellInfo(20911)),
		attribute = L["Damage Reduction"],
		attributeShort = L["Reduction"],
		mode = "Mitigation",
		interested = "tank",
		spells = {
			[20911] = {										-- Blessing of Sanctuary
				alternate = GetSpellInfo(25899),			-- Greater Blessing of Sanctuary
				class = "PALADIN",
				spec = "Protection",
				exclusive = "blessing",
				amount = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(20911),
			},
			[47517] = {										-- Grace
				class = "PRIEST",
				spec = "Discipline",
				amount = 1,
				amountType = "%",
				temp = true,
				requiredTalent = GetSpellInfo(47517),
			},
		},
	},
	[L["Haste"]] = {
		icon = select(3, GetSpellInfo(53648)),
		attribute = L["Haste"],
		mode = "DPS",
		interested = "melee,caster,healer",
		spells = {
			[53648] = {										-- Swift Retribution
				class = "PALADIN",
				spec = "Retribution",
				amountPerTalentPoint = 1,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(53648),
			},
			[48396] = {										-- Improved Moonkin Form
				class = "DRUID",
				spec = "Moonkin",
				amountPerTalentPoint = 1,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(48396),
			},
		},
	},
	[L["Haste (Melee)"]] = {
		icon = select(3, GetSpellInfo(55610)),
		attribute = L["Haste"],
		mode = "DPS",
		interested = "melee",
		spells = {
			[55610] = {										-- Improved Icy Talons
				class = "DEATHKNIGHT",
				spec = "Frost",
				amount = 20,
				amountType = "%",
				temp = true,
				requiredTalent = GetSpellInfo(55610),
			},
			[8512] = {										-- Windfury Totem
				class = "SHAMAN",
				totem = "air",
				amount = 16,
				amountType = "%",
				improvedPercentagePerTalentPoint = 2,
				maxTalentPoints = 2,
				improved = GetSpellInfo(29193),				-- Improved Windfury Totem
			},
		},
	},
	[L["Haste (Spell)"]] = {
		icon = select(3, GetSpellInfo(3738)),
		attribute = L["Haste"],
		mode = "DPS",
		interested = "caster,healer",
		spells = {
			[3738] = {										-- Wrath of Air Totem
				class = "SHAMAN",
				totem = "air",
				amount = 5,
				amountType = "%",
			},
		},
	},
	[L["Healing Received"]] = {
		icon = select(3, GetSpellInfo(33891)),
		attribute = L["Healing"],
		mode = "Healing",
		interested = "healer",
		spells = {
			[33891] = {										-- Tree of Life
				class = "DRUID",
				spec = "Restoration",
				amount = 6,
				amountType = "%",
				requiredTalent = GetSpellInfo(33891),
			},
			[20140] = {										-- Improved Devotion Aura
				class = "PALADIN",
				spec = "Protection",
				amountPerTalentPoint = 2,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(20140),
			},
		},
	},
	[L["Health"]] = {
		icon = select(3, GetSpellInfo(47440)),
		attribute = L["Health"],
		mode = "Health",
		interested = "tank,healer",
		spells = {
			[47982] = {										-- Blood Pact
				class = "WARLOCK",
				pet = L["Imp"],
				rankLevels = {4, 14, 26, 38, 50, 62, 74},
				amounts = {20, 70, 160, 270, 380, 660, 1330},
			},
			[47440] = {										-- Commanding Shout
				class = "WARRIOR",
				exclusive = "warriorbuff",
				rankLevels = {68, 74, 80},
				amounts = {1080, 1855, 2255},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 5,
				improved = GetSpellInfo(12861),				-- Commanding Presence
			},
		},
	},
	[L["Intellect"]] = {
		icon = select(3, GetSpellInfo(42995)),
		attribute = L["Intellect"],
		mode = "Stats",
		interested = "manauser",
		reportable = true,
		spells = {
			[42995] = {										-- Arcane Intellect
				alternate = GetSpellInfo(27127),			-- Arcane Brilliance
				alternateOffset = 4,						-- How many ranks to add if it's the alternate spell
				class = "MAGE",
				rankLevels = {1, 14, 28, 42, 56, 70, 80},
				amounts = {2, 7, 15, 22, 31, 40, 60},
			},
			[61024] = {										-- Dalaran Intellect
				alternate = GetSpellInfo(61316),			-- Dalaran Brilliance
				class = "MAGE",
				rankLevels = {80, 80, 80},					-- Was changed to Rank 3 (from 1) with patch 3.1 ish,
				amounts = {60, 60, 60},						-- so we'll fill in the two dummy ones else it bugs
			},
			[57567] = {										-- Fel Intelligence
				class = "WARLOCK",
				pet = "Felhunter",
				rankLevels = {32, 42, 52, 62, 72},
				amounts = {12, 18, 25, 32, 48},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 2,
				improved = GetSpellInfo(54038),				-- Improved Felhunter
				regex = 1,
			},
		},
	},
	[L["Mana Regen"]] = {
		icon = select(3, GetSpellInfo(48936)),
		attribute = L["mp5"],
		mode = "mp5",
		interested = "manauser",
		spells = {
			[48936] = {										-- Blessing of Wisdom
				alternate = GetSpellInfo(27143),			-- Greater Blessing of Wisdom
				alternateOffset = 4,						-- How many ranks to add if it's the alternate spell
				class = "PALADIN",
				exclusive = "blessing",
				rankLevels = {14, 24, 34, 44, 54, 60, 65, 71, 77},
				amounts = {10, 15, 20, 25, 30, 33, 41, 73, wowVersion >= 9868 and 92 or 91},
				improvedPercentagePerTalentPoint = 10,
				maxTalentPoints = 2,
				improved = GetSpellInfo(20245),				-- Improved Blessing of Wisdom
				regex = 1,
			},
			[58777] = {										-- Mana Spring
				class = "SHAMAN",
				totem = "water",
				rankLevels = {26, 36, 46, 56, 65, 71, 76, 80},
				amounts = {16, 21, 26, 31, 41, 73, 82, 91},
				improvedPercentagePerTalentPoint = 20/3,
				maxTalentPoints = 3,
				improved = GetSpellInfo(16187),				-- Restorative Totems
				regex = 1,
			},
		},
	},
	[L["Mana Regen (Pet)"]] = {
		icon = select(3, GetSpellInfo(44561)),
		attribute = L["Mana"],
		mode = "mp5",
		interested = "manauser",
		spells = {
			[44561] = {										-- Improved Water Elemental
				class = "MAGE",
				spec = "Frost",
				amountPerTalentPoint = 0.2,
				maxTalentPoints = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(44561),		-- Improved Water Elemental
			},
		},
	},
	[GetSpellInfo(57669)] = {								-- Replenishment
		icon = select(3, GetSpellInfo(57669)),
		id = 57669,
		attribute = L["Mana Regen"],
		mode = "mp5",
		interested = "manauser",
		spells = {
			[53292] = {										-- Hunting Party
				alternate = GetSpellInfo(57669),
				class = "HUNTER",
				spec = "Survival",
				requiredTalent = GetSpellInfo(53292),		-- Hunting Party
			},
			[31878] = {										-- Judgements of the Wise
				alternate = GetSpellInfo(57669),
				class = "PALADIN",
				spec = "Retribution",
				requiredTalent = GetSpellInfo(31878),		-- Judgements of the Wise
			},
			[48160] = {										-- Vampiric Touch
				alternate = GetSpellInfo(57669),
				class = "PRIEST",
				spec = "Shadow",
				requiredTalent = GetSpellInfo(48160),		-- Vampiric Touch
			},
		},
	},
	[L["Spell Power"]] = {
		icon = select(3, GetSpellInfo(57722)),
		attribute = L["Spell Power"],
		mode = "DPS",
		interested = "manauser",
		spells = {
			[47240] = {										-- Demonic Pact (Temp Buff)
				class = "WARLOCK",
				pet = "any",
				amountPerTalentPoint = 2,
				maxTalentPoints = 5,
				amountType = "%",
				temp = true,
				requiredTalent = GetSpellInfo(47240),		-- Demonic Pact
			},
			[58656] = {										-- Flametongue Totem
				class = "SHAMAN",
				totem = "fire",
				rankLevels = {28, 38, 48, 58, 67, 71, 75, 80},
				amounts = {25, 37, 49, 62, 73, 106, 122, 144},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 3,
				improved = GetSpellInfo(52456),				-- Enhancing Totems
				regex = true,
			},
			[57722] = {										-- Totem of Wrath
				class = "SHAMAN",
				totem = "fire",
				rankLevels = {50, 60, 70, 80},
				amounts = {100, 120, 140, 280},
				requiredTalent = GetSpellInfo(57722),		-- Totem of Wrath
			},
			--[33182] = {										-- Improved Divine Spirit
			--	class = "PRIEST",
			--	spec = "Discipline",
			--	amounts = {17, 23, 33, 40, 50, 80},			-- TODO: This currently assumes 2/2 points in Improved Divine Spirit
			--	requiredTalent = GetSpellInfo(33182),
			--	regex = 1,
			--},
		},
	},
	[L["Spirit"]] = {
		icon = select(3, GetSpellInfo(48073)),
		attribute = L["Spirit"],
		mode = "Stats",
		interested = "healer",
		reportable = true,
		spells = {
			[48073] = {										-- Divine Spirit
				alternate = GetSpellInfo(32999),			-- Prayer of Spirit
				alternateOffset = 3,						-- How many ranks to add if it's the alternate spell
				class = "PRIEST",
				rankLevels = {30, 40, 50, 60, 70, 80},
				amounts = {17, 23, 33, 40, 50, 80},
				regex = true,
			},
			[57567] = {										-- Fel Intelligence
				class = "WARLOCK",
				pet = "Felhunter",
				rankLevels = {32, 42, 52, 62, 72},
				amounts = {18, 26, 32, 40, 64},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 2,
				improved = GetSpellInfo(54038),				-- Improved Felhunter
				regex = true,
			},
		},
	},
	[L["Stamina"]] = {
		icon = select(3, GetSpellInfo(48161)),
		attribute = L["Stamina"],
		mode = "Stats",
		interested = "tank,healer",
		reportable = true,
		spells = {
			[48161] = {										-- Power Word: Fortitude
				alternate = GetSpellInfo(25392),			-- Prayer of Fortitude
				alternateOffset = 4,						-- How many ranks to add if it's the alternate spell
				class = "PRIEST",
				rankLevels = {1, 12, 24, 36, 48, 60, 70, 80},
				amounts = {3, 8, 20, 32, 43, 54, 79, 165},
				improvedPercentagePerTalentPoint = 15,
				maxTalentPoints = 2,
				improved = GetSpellInfo(14767),				-- Improved Power Word: Fortitude
				regex = true,
			},
		},
	},
	[L["Stat Add"]] = {
		icon = select(3, GetSpellInfo(48469)),
		attribute = L["Stats"],
		mode = "Stats",
		reportable = true,
		spells = {
			[48469] = {										-- Mark of the Wild
				alternate = GetSpellInfo(26991),			-- Gift of the Wild
				alternateOffset = 5,						-- How many ranks to add if it's the alternate spell
				class = "DRUID",
				rankLevels = {1, 10, 20, 30, 40, 50, 60, 70, 80},
				amounts = {0, 2, 4, 6, 8, 10, 12, 14, 37},
				improvedPercentagePerTalentPoint = 20,
				maxTalentPoints = 2,
				improved = GetSpellInfo(17051),				-- Improved Mark of the Wild
				regex = "TALENT",							-- Change all figures in tooltip by amount given on talents
			},
		},
	},
	[L["Stat Multiplier"]] = {
		icon = select(3, GetSpellInfo(20217)),
		attribute = L["Stats"],
		mode = "Stats",
		reportable = true,
		spells = {
			[20217] = {										-- Blessing of Kings
				alternate = GetSpellInfo(25898),			-- Greater Blessing of Kings
				class = "PALADIN",
				exclusive = "blessing",
				amount = 10,
				amountType = "%",
			},
		},
	},
	[L["Strength & Agility"]] = {
		icon = select(3, GetSpellInfo(58643)),
		attribute = L["Str & Agi"],
		mode = "DPS",
		interested = "melee,tank",
		spells = {
			[58646] = {										-- Strength of Earth (not Strength of Earth Totem, which is different)
				class = "SHAMAN",
				totem = "earth",
				totemBuffName = GetSpellInfo(58646),
				rankLevels = {10, 24, 38, 52, 60, 65, 75, 80},
				amounts = {10, 20, 36, 61, 77, 86, 115, 155},
				improvedPercentagePerTalentPoint = 5,
				maxTalentPoints = 3,
				improved = GetSpellInfo(52456),				-- Enhancing Totems
				regex = true,
			},
			[57623] = {										-- Horn of Winter
				class = "DEATHKNIGHT",
				rankLevels = {65, 75},
				amounts = {86, 155},
				regex = true,
			},
		},
	},
}

Utopia.debuffs = {
	[L["Armor (Major)"]] = {
		icon = select(3, GetSpellInfo(47467)),
		attribute = L["Armor"],
		interested = "melee,tank",
		spells = {
			[8647] = {										-- Expose Armor
				class = "ROGUE",
				amount = 20,
				amountType = "%",
			},
			[47467] = {										-- Sunder Armor
				class = "WARRIOR",
				amountPerStack = 4,
				amountType = "%",
				maxStacks = 5,
			},
		},
	},
	[L["Armor (Minor)"]] = {
		icon = select(3, GetSpellInfo(770)),
		attribute = L["Armor"],
		interested = "melee,tank",
		reportable = true,
		spells = {
			[16231] = {										-- Curse of Recklessness
				class = "WARLOCK",
				amount = 5,
				amountType = "%",
				exclusive = "curse",
			},
			[770] = {										-- Faerie Fire
				class = "DRUID",
				amount = 5,
				amountType = "%",
			},
			[16857] = {										-- Faerie Fire (Feral)
				class = "DRUID",
				amount = 5,
				amountType = "%",
			},
			[56631] = {										-- Sting
				class = "HUNTER",
				amount = 5,
				amountType = "%",
				pet = L["Wasp"]
			},
		},
	},
	[L["Attack Power"]] = {
		icon = select(3, GetSpellInfo(47437)),
		attribute = L["Attack Power"],
		interested = "melee,tank",
		spells = {
			[50511] = {										-- Curse of Weakness
				class = "WARLOCK",
				rankLevels = {4, 12, 22, 32, 42, 52, 61, 69, 71},
				amounts = {21, 41, 64, 82, 123, 163, 257, 350, 478},
				spec = "Affliction",
				improvedPercentagePerTalentPoint = 10,
				maxTalentPoints = 2,
				improved = GetSpellInfo(18179),				-- Frailty
				exclusive = "curse",
			},
			[48560] = {										-- Demoralizing Roar
				class = "DRUID",
				rankLevels = {10, 20, 32, 42, 52, 62, 71, 77},
				amounts = {31, 51, 68, 103, 138, 220, 299, 408},
				spec = "Feral",
				improvedPercentagePerTalentPoint = 8,
				maxTalentPoints = 5,
				improved = GetSpellInfo(16862)				-- Feral Aggression
			},
			[47437] = {										-- Demoralizing Shout
				class = "WARRIOR",
				rankLevels = {14, 24, 34, 44, 54, 62, 70, 79},
				amounts = {35, 55, 70, 105, 140, 220, 300, 418},
				spec = "Fury",
				improvedPercentagePerTalentPoint = 8,
				maxTalentPoints = 5,
				improved = GetSpellInfo(12879)				-- Improved Demoralizing Shout
			},
			[55487] = {										-- Demoralizing Screech
				class = "HUNTER",
				rankLevels = {1, 16, 32, 48, 64, 80},
				amounts = {25, 45, 70, 100, 210, 410},
				pet = L["Bird of Prey"],
				improved = GetSpellInfo(12879)
			},
		},
	},
	[L["Bleed Damage"]] = {
		icon = select(3, GetSpellInfo(48564)),
		interested = "melee,tank",
		spells = {
			[33917] = {										-- Mangle
				class = "DRUID",
				amount = 30,
				amountType = "%",
				spec = "Feral",
				requiredTalent = GetSpellInfo(33917),		-- Mangle
			},
			[46855] = {										-- Trauma
				class = "WARRIOR",
				amount = 30,
				amountType = "%",
				spec = "Arms",
				requiredTalent = GetSpellInfo(46855),		-- Trauma
			},
		},
	},
	[L["Cast Speed Slow"]] = {
		icon = select(3, GetSpellInfo(5761)),
		interested = "tank",
		spells = {
			[11719] = {										-- Curse of Tongues
				class = "WARLOCK",
				rankLevels = {26, 50},
				amounts = {25, 30},
				amountType = "%",
				exclusive = "curse",
			},
			[5761] = {										-- Mind-numbing Poison
				class = "ROGUE",
				amount = 30,
				amountType = "%"
			},
			[58611] = {										-- Lava Breath
				class = "HUNTER",
				amount = 25,
				amountType = "%",
				spec = "Beastmaster",
				requiredTalent = GetSpellInfo(53270),		-- Beast Mastery
				pet = L["Core Hound"]
			},
			[31589] = {										-- Slow
				class = "MAGE",
				spec = "Arcane",
				requiredTalent = GetSpellInfo(31589),		-- Slow
				amount = 30,
				amountType = "%"
			},
		},
	},
	[L["Critical Strike Chance Taken"]] = {
		icon = select(3, GetSpellInfo(58410)),
		interested = "melee,tank,caster",
		spells = {
			[20337] = {										-- Heart of the Crusader
				class = "PALADIN",
				spec = "Retribution",
				maxTalentPoints = 3,
				amountPerTalentPoint = 1,
				amountType = "%",
				requiredTalent = GetSpellInfo(20337)		-- Heart of the Crusader
			},
			[58410] = {										-- Master Poisoner
				class = "ROGUE",
				spec = "Assassination",
				maxTalentPoints = 3,
				amountPerTalentPoint = 1,
				amountType = "%",
				requiredTalent = GetSpellInfo(58410)		-- Master Poisoner
			},
			[30706] = {										-- Totem of Wrath
				class = "SHAMAN",
				totem = "fire",
				spec = "Elemental",
				amount = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(30706)		-- Totem of Wrath
			},
		},
	},
	[L["Healing Taken"]] = {
		icon = select(3, GetSpellInfo(47486)),
		interested = "tank",
		spells = {
			[49050] = {										-- Aimed Shot
				class = "HUNTER",
				spec = "Marksmanship",
				amount = 50,
				amountType = "%",
				requiredTalent = GetSpellInfo(49050)		-- Aimed Shot
			},
			[46911] = {										-- Furious Attacks
				class = "WARRIOR",
				spec = "Fury",
				amountPerStack = 25,
				maxStacks = 2,
				amountType = "%",
				requiredTalent = GetSpellInfo(46911)		-- Furious Attacks
			},
			[47486] = {										-- Mortal Strike
				class = "WARRIOR",
				spec = "Arms",
				amount = 50,
				amountType = "%",
				requiredTalent = GetSpellInfo(47486)		-- Mortal Strike
			},
			[13218] = {										-- Wound Poison (unranked name)
				class = "ROGUE",
				amount = 50,
				amountType = "%"
			},
		},
	},
	[L["Health Restore"]] = {
		icon = select(3, GetSpellInfo(20271)),
		interested = "tank,healer",
		reportable = true,
		spells = {
			[20271] = {										-- Judgement of Light
				class = "PALADIN",
				exclusive = "judge",
			},
		},
	},
	[L["Mana Restore"]] = {
		icon = select(3, GetSpellInfo(53408)),
		interested = "manauser",
		reportable = true,
		spells = {
			[53408] = {										-- Judgement of Wisdom
				class = "PALADIN",
				exclusive = "judge",
			},
		},		
	},
	[L["Melee Attack Speed Slow"]] = {
		icon = select(3, GetSpellInfo(47502)),
		interested = "tank",
		spells = {
			[55095] = {										-- Frost Fever
				class = "DEATHKNIGHT",
				spec = "Frost",
				amount = 14,
				amountType = "%",
				source = GetSpellInfo(49909),				-- Icy Touch
				improvedAmountPerTalentPoint = 2,
				maxTalentPoints = 3,
				improved = GetSpellInfo(51456)				-- Improved Icy Touch
			},
			[48485] = {										-- Infected Wounds
				class = "DRUID",
				spec = "Feral",
				maxStacks = 2,
				amountPerTalentPoint = 10/3,
				amountType = "%",
				maxTalentPoints = 3,
				requiredTalent = GetSpellInfo(48485)		-- Infected Wounds
			},
			[53696] = {										-- Judgements of the Just
				class = "PALADIN",
				spec = "Protection",
				amountPerTalentPoint = 10,
				amountType = "%",
				maxTalentPoints = 2,
				requiredTalent = GetSpellInfo(53696)		-- Judgements of the Just
			},
			[47502] = {										-- Thunder Clap
				class = "WARRIOR",
				spec = "Protection",
				amount = 10,
				amountType = "%",
				maxTalentPoints = 3,
				improvedAmountPerTalentPoint = 10/3,
				improved = GetSpellInfo(12666)				-- Improved Thunder Clap
			},
		},
	},
	[L["Melee Hit Chance Reduction"]] = {
		icon = select(3, GetSpellInfo(48468)),
		interested = "tank",
		spells = {
			[48468] = {										-- Insect Swarm
				class = "DRUID",
				amount = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(48468),
			},
			[3043] = {										-- Scorpid Sting
				class = "HUNTER",
				amount = 3,
				amountType = "%"
			},
		},
	},
	[L["Physical Vulnerability"]] = {
		icon = select(3, GetSpellInfo(58413)),
		interested = "tank,melee",
		spells = {
			[29859] = {										-- Blood Frenzy
				class = "WARRIOR",
				spec = "Arms",
				amountPerTalentPoint = 2,
				amountType = "%",
				maxTalentPoints = 2,
				requiredTalent = GetSpellInfo(29859)		-- Blood Frenzy
			},
			[58413] = {										-- Savage Combat
				class = "ROGUE",
				spec = "Combat",
				amountPerTalentPoint = 2,
				amountType = "%",
				maxTalentPoints = 2,
				requiredTalent = GetSpellInfo(58413)		-- Savage Combat
			},
		},
	},
	[L["Spell Critical Strike Chance"]] = {
		icon = select(3, GetSpellInfo(12873)),
		interested = "caster",
		spells = {
			[12873] = {										-- Improved Scorch
				class = "MAGE",
				spec = "Fire",
				amountPerStack = 1,
				maxStacks = 5,
				amountType = "%",
				maxTalentPoints = 5,
				requiredTalent = GetSpellInfo(12873)
			},
			[28593] = {										-- Winter's Chill
				class = "MAGE",
				spec = "Frost",
				amountPerStack = 1,
				maxStacks = 5,
				amountType = "%",
				maxTalentPoints = 5,
				requiredTalent = GetSpellInfo(28593)
			},
			[17800] = {										-- Shadow Mastery
				class = "WARLOCK",
				spec = "Destruction",
				amount= 5,
				amountType = "%",
				maxTalentPoints = 5,
				requiredTalent = GetSpellInfo(17803)		-- Improved Shadow Bolt
			},
		},
	},
	[L["Spell Damage Taken"]] = {
		icon = select(3, GetSpellInfo(47865)),
		interested = "caster",
		spells = {
			[47865] = {										-- Curse of the Elements
				class = "WARLOCK",
				spec = "Affliction",
				amount = 13,
				amountType = "%",
				exclusive = "curse",
			},
			[48511] = {										-- Earth and Moon
				class = "DRUID",
				spec = "Balance",
				amountPerTalentPoint = 13/3,
				amountType = "%",
				maxTalentPoints = 3,
				requiredTalent = GetSpellInfo(48511)		-- Earth and Moon
			},
			[51735] = {										-- Ebon Plague
				class = "DEATHKNIGHT",
				spec = "Unholy",
				amountPerTalentPoint = 13/3,
				amountType = "%",
				maxTalentPoints = 3,
				requiredTalent = GetSpellInfo(51161)		-- Ebon Plaguebringer
			},
		},
	},
	[L["Spell Hit Chance Taken"]] = {
		icon = select(3, GetSpellInfo(33193)),
		interested = "caster",
		spells = {
			[33602] = {										-- Improved Faerie Fire
				class = "DRUID",
				spec = "Balance",
				amountPerTalentPoint = 1,
				amountType = "%",
				maxTalentPoints = 3,
				requiredTalent = GetSpellInfo(33602)		-- Improved Faerie Fire
			},
			[33193] = {										-- Misery
				class = "PRIEST",
				spec = "Shadow",
				amount = 3,
				amountType = "%",
				requiredTalent = GetSpellInfo(33193)		-- Misery
			},
		},
	},
}

-- Buffs we want to enhance the tooltips for, but are not showing them in the icon grid
Utopia.otherBuffs = {
	[54043] = {									-- Retribution Aura
		rankLevels = {16, 26, 36, 46, 56, 66, 76},
		amounts = {10, 18, 27, 37, 48, 62, 112},
		exclusive = "aura",
		improved = GetSpellInfo(31869),			-- Sanctified Retribution
		improvedPercentagePerTalentPoint = 50,
		maxTalentPoints = 1,
		regex = true,
	},
	[19746] = {									-- Concentration Aura
		amount = 35,
		amountType = "%",
		exclusive = "aura",
		improved = GetSpellInfo(20254),
		improvedPercentagePerTalentPoint = 5,
		maxTalentPoints = 3,
		regex = true,
	},
	[48943] = {exclusive = "aura"},				-- Shadow Resistance Aura
	[48945] = {exclusive = "aura"},				-- Frost Resistance Aura
	[48947] = {exclusive = "aura"},				-- Fire Resistance Aura
}

-- Mapping spells. These don't actually appear on the target, but are
-- 'hidden' auras that we have to infer, often given as an additional
-- effect to a spell.  (With the exception of Wound Poison which is a
-- straight mapping to cope with the varying names)

Utopia.mapping = {
	[GetSpellInfo(13218)] = {					-- Wound Poison (unranked name)
		class = "ROGUE",
		type = "map",
		list = {
			[GetSpellInfo(13225)] = true,		-- Wound Poison II
			[GetSpellInfo(13226)] = true,		-- Wound Poison III
			[GetSpellInfo(13227)] = true,		-- Wound Poison IV
			[GetSpellInfo(27188)] = true,		-- Wound Poison V
			[GetSpellInfo(57977)] = true,		-- Wound Poison VI
			[GetSpellInfo(57978)] = true,		-- Wound Poison VII
		},
	},

	[GetSpellInfo(58410)] = {					-- Master Poisoner
		class = "ROGUE",
		requiredTalent = GetSpellInfo(58410),	-- Master Poisoner
		type = "trigger",
		list = {
			[GetSpellInfo(3408)] = true,		-- Crippling Poison
			[GetSpellInfo(41190)] = true,		-- Mind-numbing Poison
			[GetSpellInfo(13218)] = true,		-- Wound Poison
			[GetSpellInfo(2823)] = true,		-- Deadly Poison
		},
	},

	[GetSpellInfo(53696)] = {					-- Judgements of the Just
		class = "PALADIN",
		requiredTalent = GetSpellInfo(53696),	-- Judgements of the Just
		type = "trigger",
		list = {
			[GetSpellInfo(53408)] = true,		-- Judgement of Wisdom
			[GetSpellInfo(20271)] = true,		-- Judgement of Light
			[GetSpellInfo(53407)] = true,		-- Judgement of Justice
		},
	},

	[GetSpellInfo(20140)] = {					-- Improved Devotion Aura
		class = "PALADIN",
		requiredTalent = GetSpellInfo(20140),	-- Improved Devotion Aura
		type = "trigger",
		list = {
			[GetSpellInfo(48942)] = true,		-- Devotion Aura
		},
	},

	--[GetSpellInfo(33182)] = {					-- Improved Divine Spirit
	--	class = "PRIEST",
	--	requiredTalent = GetSpellInfo(33182),	-- Improved Divine Spirit
	--	type = "trigger",
	--	list = {
	--		[GetSpellInfo(48073)] = true,		-- Divine Spirit
	--	},
	--},

	[GetSpellInfo(53648)] = {					-- Swift Retribution
		class = "PALADIN",
		requiredTalent = GetSpellInfo(53648),	-- Swift Retribution
		type = "trigger",
		list = {
			[GetSpellInfo(48942)] = true,		-- Devotion Aura
			[GetSpellInfo(54043)] = true,		-- Retribution Aura
			[GetSpellInfo(19746)] = true,		-- Concentration Aura
			[GetSpellInfo(48943)] = true,		-- Shadow Resistance Aura
			[GetSpellInfo(48945)] = true,		-- Frost Resistance Aura
			[GetSpellInfo(48947)] = true,		-- Fire Resistance Aura
			[GetSpellInfo(32223)] = true,		-- Crusader Aura
		},
	},

	[GetSpellInfo(31869)] = {					-- Sanctified Retribution
		class = "PALADIN",
		requiredTalent = GetSpellInfo(31869),	-- Sanctified Retribution
		type = "trigger",
		list = {
			[GetSpellInfo(48942)] = true,		-- Devotion Aura
			[GetSpellInfo(54043)] = true,		-- Retribution Aura
			[GetSpellInfo(19746)] = true,		-- Concentration Aura
			[GetSpellInfo(48943)] = true,		-- Shadow Resistance Aura
			[GetSpellInfo(48945)] = true,		-- Frost Resistance Aura
			[GetSpellInfo(48947)] = true,		-- Fire Resistance Aura
			[GetSpellInfo(32223)] = true,		-- Crusader Aura
		},
	},

	[GetSpellInfo(33602)] = {					-- Improved Faerie Fire
		class = "DRUID",
		requiredTalent = GetSpellInfo(33602),	-- Improved Faerie Fire
		type = "trigger",
		list = {
			[GetSpellInfo(770)] = true,			-- Faerie Fire
			[GetSpellInfo(16857)] = true,		-- Faerie Fire (Feral)
		},
	},

	[GetSpellInfo(48396)] = {					-- Improved Moonkin Form
		class = "DRUID",
		requiredTalent = GetSpellInfo(48396),	-- Moonkin Aura
		list = {
			[GetSpellInfo(24907)] = true,		-- Moonkin Aura
		}
	},

	[GetSpellInfo(33917)] = {
		class = "DRUID",
		requiredTalent = GetSpellInfo(33917),
		type = "trigger",
		list = {
			[GetSpellInfo(48564)] = true,		-- Mangle - Bear
			[GetSpellInfo(48566)] = true,		-- Mangle - Cat
		},
	},
}

for Type,data in pairs(Utopia.debuffs) do
	assert(type(Type) == "string")
	assert(data.spells)
	for spellName,info in pairs(data.spells) do
		assert(info.class)
		assert(type(spellName) == "number")
		if (info.amount) then
			assert(type(info.amount) == "number")
		elseif (info.amountsPerStack) then
			assert(info.rankLevels)
			assert(#info.amountsPerStack == #info.rankLevels)
		elseif (info.amounts) then
			assert(type(info.amounts) == "table")
			assert(info.rankLevels)
			assert(#info.amounts == #info.rankLevels)
		end
	end
end

for name,data in pairs(Utopia.mapping) do
	assert(data.class)
	assert(data.list)
	for spellName,True in pairs(data.list) do
		assert(type(spellName) == "string")
		assert(True)
	end
end	
